==> jsmex-trader-sdk-1.1.26.tgz: package/lib/trader/BaseStrategyTrader.js <==
const {
  DepthDTO,
  KlineDTOArray,
  TransactionDTOArray,
  OrderDTOArray,
  BalanceDTO
} = require('../data')
global.BigNumber = global.BigNumber || require('../../static_dependencies/bignumber/bignumber.min.js')

/**
* BaseStrategyTrader class
*
* @author jsb4ch <jsb4ch@jsmex.io>
* @since 1.0.0
*/
const BaseStrategyTrader = (BaseTrader) => class extends BaseTrader {
  constructor (options) {
    super(options)
    this._depth = {}
    this._kline = {}
    this._transactions = []
    this._orders = []
    this._balance = {}
    this.defaultSymbol = this.options.defaultSymbol
  }

  /**
    * BaseStrategyTrader defaultSymbol getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    */
  get default () {
    return this.defaultSymbol
  }

  set default (defaultSymbol) {
    this.defaultSymbol = defaultSymbol
  }

  /**
    * BaseStrategyTrader _onInit
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onInit () {
    await this.getPublicData()
    // getPrivateData
    if (typeof this.onInit === 'function') {
      await this.onInit()
    }
  }

  /**
    * BaseStrategyTrader _onTick
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onTick () {
    if (typeof this.onTick === 'function') {
      await this.onTick()
    } else {
      throw new Error('onTick must be overridden')
    }
  }

  /**
    * BaseStrategyTrader _onExit
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onExit () {
    if (typeof this.onExit === 'function') {
      await this.onExit()
    }
  }

  /**
    * BaseStrategyTrader getPublicData
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} fetchDepthParams
    * @param {KlineDTO} fetchOHLCVParams
    * @returns {Promise<Boolean>}
    */
  async getPublicData (fetchDepthParams, fetchOHLCVParams) {
    let result = await Promise.all([
      this.fetchDepth(fetchDepthParams),
      this.fetchOHLCV(fetchOHLCVParams)
    ])
    return this.update({
      depthDTO: result[0],
      klineDTOArray: result[1]
    })
  }

  /**
    * BaseStrategyTrader getPrivateData
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} fetchDepthParams
    * @param {KlineDTO} fetchOHLCVParams
    * @returns {Promise<Boolean>}
    */
  async getPrivateData (fetchOrdersParams, fetchTransactionsParams) {
    let result = await Promise.all([
      this.fetchOrders(this.options.ccxt.fetchOrders),
      this.fetchTransactions(this.options.ccxt.fetchTransactions),
      this.balance()
    ])
    return this.update({
      orderDTOArray: result[0],
      transactionDTOArray: result[1],
      balanceDTO: result[2]
    })
  }

  /**
    * BaseStrategyTrader update method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} depthDTO
    * @param {KlineDTO} klineDTO
    * @returns {Boolean}
    */
  update ({ depthDTO, klineDTOArray, orderDTOArray, transactionDTOArray, balanceDTO } = {}) {
    if (depthDTO && DepthDTO.isValid(depthDTO)) {
      this._depth[depthDTO.symbolId] = depthDTO
    }

    if (klineDTOArray && KlineDTOArray.isValid(klineDTOArray) && klineDTOArray.length) {
      const symbolId = klineDTOArray[0].symbolId
      this._kline[symbolId] = klineDTOArray
    }

    if (transactionDTOArray && TransactionDTOArray.isValid(transactionDTOArray)) {
      if (this._transactions) {
        this._transactions = this._transactions.concat(transactionDTOArray)
        this._transactions = TransactionDTOArray.removeDuplicates(this._transactions, 'id')
      } else {
        this._transactions = transactionDTOArray
      }
    }

    if (orderDTOArray && OrderDTOArray.isValid(orderDTOArray)) {
      if (this._orders) {
        this._orders = this._orders.concat(orderDTOArray)
        this._orders = OrderDTOArray.removeDuplicates(this._orders, 'id')
      } else {
        this._orders = orderDTOArray
      }
    }

    if (balanceDTO && BalanceDTO.isValid(balanceDTO)) {
      this._balance = depthDTO
    }
    return this.validate()
  }

  /**
    * BaseStrategyTrader length getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Number}
    */
  get length () {
    return Object.keys(this._kline || {}).length || Object.keys(this._depth || {}).length
  }

  /**
    * BaseStrategyTrader keys getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Number}
    */
  get keys () {
    return Object.keys(this._kline || {}) || Object.keys(this._depth || {})
  }

  /**
    * BaseStrategyTrader validate method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Boolean}
    */
  validate () {
    for (const currentDepth in this._depth) {
      if (!DepthDTO.isValid(this._depth[currentDepth])) {
        const error = { code: 2017, isCritical: true, message: 'depth is not instance of DepthDTO', info: this._depth }
        throw error
      }
    }

    for (const currentKline in this._kline) {
      if (!KlineDTOArray.isValid(this._kline[currentKline])) {
        const error = { code: 2017, isCritical: true, message: 'kline is not instance of KlineDTOArray', info: this._kline }
        throw error
      }
    }

    if (!TransactionDTOArray.isValid(this._transactions)) {
      const error = { code: 2017, isCritical: true, message: '_transactions is not instance of KlineDTOArray', info: this._kline }
      throw error
    }

    if (!OrderDTOArray.isValid(this._orders)) {
      const error = { code: 2017, isCritical: true, message: '_orders is not instance of KlineDTOArray', info: this._kline }
      throw error
    }

    // if (!BalanceDTO.isValid(this._balance)) {
    //   const error = { code: 2017, isCritical: true, message: 'kline is not instance of KlineDTOArray', info: this._kline }
    //   throw error
    // }

    return true
  }

  /**
    * BaseStrategyTrader depth method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} symbol
    * @returns {DepthDTO}
    */
  depth (symbol = this.defaultSymbol) {
    return this._depth[symbol]
  }

  /**
    * BaseStrategyTrader open method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  open (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('open', index, symbol)
  }

  /**
    * BaseStrategyTrader close method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  close (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('close', index, symbol)
  }

  /**
    * BaseStrategyTrader high method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  high (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('high', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  low (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('low', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  volume (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('volume', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {Number}
    */
  time (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('time', index, symbol)
  }

  // /**
  //   * BaseStrategyTrader RSI method
  //   *
  //   * @author jsb4ch <jsb4ch@jsmex.io>
  //   * @since 1.0.0
  //   * @param index number
  //   * @param symbol string
  //   * @returns number
  //   */
  // RSI (index = 0, symbol = this.defaultSymbol) {}

  /**
    * BaseStrategyTrader getKlineValue method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} symbol
    * @returns {Number}
    */
  klineLength (symbol = this.defaultSymbol) {
    const kline = this._kline[symbol]
    if (!kline) {
      return null
    }
    return kline.length
  }

  /**
    * BaseStrategyTrader getKlineValue method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} propertyName
    * @param {Number} index
    * @param {String} symbol
    * @returns {KlineDTO}
    */
  getKlineValue (propertyName, index = 0, symbol = this.defaultSymbol) {
    const kline = this._kline[symbol]
    if (!kline || !kline.length) {
      return null
    }
    if (index < 0) {
      index = kline.length + index > 0 ? kline.length + index : kline.length - 1
    }
    return global.BigNumber(kline[index][propertyName])
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {BalanceDTO}
    */
  get balance () {
    return this._balance
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {TransactionDTOArray}
    */
  get transactions () {
    return this._transactions
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {OrderDTOArray}
    */
  get orders () {
    return this._orders
  }
}

module.exports = BaseStrategyTrader

==> jsmex-trader-sdk-1.1.50.tgz: package/lib/trader/BaseStrategyTrader.js <==
const {
  DepthDTO,
  KlineDTOArray,
  TransactionDTOArray,
  OrderDTOArray
  // BalanceDTO
} = require('../data')
global.BigNumber = global.BigNumber || require('../../static_dependencies/bignumber/bignumber.min.js')

/**
* BaseStrategyTrader class
*
* @author jsb4ch <jsb4ch@jsmex.io>
* @since 1.0.0
*/
const BaseStrategyTrader = (BaseTrader) => class extends BaseTrader {
  constructor (options) {
    super(options)
    this._depth = {}
    this._kline = {}
    this._transactions = []
    this._orders = []
    this._balance = {}
    this.defaultSymbol = this.options.defaultSymbol
  }

  /**
    * BaseStrategyTrader defaultSymbol getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    */
  get default () {
    return this.defaultSymbol
  }

  set default (defaultSymbol) {
    this.defaultSymbol = defaultSymbol
  }

  /**
    * BaseStrategyTrader _onInit
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onInit () {
    await this.getPublicData()
    await this.getPrivateData()

    if (typeof this.onInit === 'function') {
      await this.onInit()
    }
  }

  /**
    * BaseStrategyTrader _onTick
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onTick () {
    if (typeof this.onTick === 'function') {
      await this.onTick()
    } else {
      throw new Error('onTick must be overridden')
    }
  }

  /**
    * BaseStrategyTrader _onExit
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Promise}
    */
  async _onExit () {
    if (typeof this.onExit === 'function') {
      await this.onExit()
    }
  }

  /**
    * BaseStrategyTrader getPublicData
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} fetchDepthParams
    * @param {KlineDTO} fetchOHLCVParams
    * @returns {Promise<Boolean>}
    */
  async getPublicData (fetchDepthParams, fetchOHLCVParams) {
    let result = await Promise.all([
      this.fetchDepth(fetchDepthParams),
      this.fetchOHLCV(fetchOHLCVParams)
    ])
    return this.update({
      depthDTO: result[0],
      klineDTOArray: result[1]
    })
  }

  /**
    * BaseStrategyTrader getPrivateData
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} fetchDepthParams
    * @param {KlineDTO} fetchOHLCVParams
    * @returns {Promise<Boolean>}
    */
  async getPrivateData (fetchOrdersParams, fetchTransactionsParams) {
    let result = await Promise.all([
      this.fetchOrders(this.options.ccxt.fetchOrders),
      this.fetchTransactions(this.options.ccxt.fetchTransactions),
      this.fetchBalance()
    ])
    return this.update({
      orderDTOArray: result[0],
      transactionDTOArray: result[1],
      balanceDTO: result[2]
    })
  }

  /**
    * BaseStrategyTrader update method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {DepthDTO} depthDTO
    * @param {KlineDTO} klineDTO
    * @returns {Boolean}
    */
  update ({ depthDTO, klineDTOArray, orderDTOArray, transactionDTOArray, balanceDTO } = {}) {
    if (depthDTO && DepthDTO.isValid(depthDTO)) {
      this._depth[depthDTO.symbolId] = depthDTO
    }

    if (klineDTOArray && KlineDTOArray.isValid(klineDTOArray) && klineDTOArray.length) {
      const symbolId = klineDTOArray[0].symbolId
      this._kline[symbolId] = klineDTOArray
    }

    if (transactionDTOArray && TransactionDTOArray.isValid(transactionDTOArray)) {
      if (this._transactions) {
        this._transactions = this._transactions.concat(transactionDTOArray)
        this._transactions = TransactionDTOArray.removeDuplicates(this._transactions, 'id')
      } else {
        this._transactions = transactionDTOArray
      }
    }

    if (orderDTOArray && OrderDTOArray.isValid(orderDTOArray)) {
      if (this._orders) {
        this._orders = this._orders.concat(orderDTOArray)
        this._orders = OrderDTOArray.removeDuplicates(this._orders, 'id')
      } else {
        this._orders = orderDTOArray
      }
    }

    // if (balanceDTO && BalanceDTO.isValid(balanceDTO)) {
    //   this._balance = balanceDTO
    // }
    this._balance = balanceDTO
    return this.validate()
  }

  /**
    * BaseStrategyTrader length getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Number}
    */
  get length () {
    return Object.keys(this._kline || {}).length || Object.keys(this._depth || {}).length
  }

  /**
    * BaseStrategyTrader keys getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Number}
    */
  get keys () {
    return Object.keys(this._kline || {}) || Object.keys(this._depth || {})
  }

  /**
    * BaseStrategyTrader validate method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @returns {Boolean}
    */
  validate () {
    for (const currentDepth in this._depth) {
      if (!DepthDTO.isValid(this._depth[currentDepth])) {
        const error = { code: 2017, isCritical: true, message: 'depth is not instance of DepthDTO', info: this._depth }
        throw error
      }
    }

    for (const currentKline in this._kline) {
      if (!KlineDTOArray.isValid(this._kline[currentKline])) {
        const error = { code: 2017, isCritical: true, message: 'kline is not instance of KlineDTOArray', info: this._kline }
        throw error
      }
    }

    if (!TransactionDTOArray.isValid(this._transactions)) {
      const error = { code: 2017, isCritical: true, message: '_transactions is not instance of KlineDTOArray', info: this._kline }
      throw error
    }

    if (!OrderDTOArray.isValid(this._orders)) {
      const error = { code: 2017, isCritical: true, message: '_orders is not instance of KlineDTOArray', info: this._kline }
      throw error
    }

    // if (!BalanceDTO.isValid(this._balance)) {
    //   const error = { code: 2017, isCritical: true, message: 'kline is not instance of KlineDTOArray', info: this._kline }
    //   throw error
    // }

    return true
  }

  /**
    * BaseStrategyTrader depth method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} symbol
    * @returns {DepthDTO}
    */
  depth (symbol = this.defaultSymbol) {
    return this._depth[symbol]
  }

  /**
    * BaseStrategyTrader open method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  open (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('open', index, symbol)
  }

  /**
    * BaseStrategyTrader close method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  close (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('close', index, symbol)
  }

  /**
    * BaseStrategyTrader high method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  high (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('high', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  low (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('low', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {String}
    */
  volume (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('volume', index, symbol)
  }

  /**
    * BaseStrategyTrader low method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {Number} index
    * @param {String} symbol
    * @returns {Number}
    */
  time (index = 0, symbol = this.defaultSymbol) {
    return this.getKlineValue('time', index, symbol)
  }

  // /**
  //   * BaseStrategyTrader RSI method
  //   *
  //   * @author jsb4ch <jsb4ch@jsmex.io>
  //   * @since 1.0.0
  //   * @param index number
  //   * @param symbol string
  //   * @returns number
  //   */
  // RSI (index = 0, symbol = this.defaultSymbol) {}

  /**
    * BaseStrategyTrader getKlineValue method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} symbol
    * @returns {Number}
    */
  klineLength (symbol = this.defaultSymbol) {
    const kline = this._kline[symbol]
    if (!kline) {
      return null
    }
    return kline.length
  }

  /**
    * BaseStrategyTrader getKlineValue method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.0
    * @param {String} propertyName
    * @param {Number} index
    * @param {String} symbol
    * @returns {KlineDTO}
    */
  getKlineValue (propertyName, index = 0, symbol = this.defaultSymbol) {
    const kline = this._kline[symbol]
    if (!kline || !kline.length) {
      return null
    }
    if (index < 0) {
      index = kline.length + index > 0 ? kline.length + index : kline.length - 1
    }
    return global.BigNumber(kline[index][propertyName])
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {BalanceDTO}
    */
  get balance () {
    return this._balance
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {TransactionDTOArray}
    */
  get transactions () {
    return this._transactions
  }

  /**
    * BaseStrategyTrader balance getter
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.1.0
    * @returns {OrderDTOArray}
    */
  get orders () {
    return this._orders
  }
}

module.exports = BaseStrategyTrader
